<?php

require "../../../init.php";

$whmcs->load_function("gateway");
$whmcs->load_function("invoice");

require_once dirname(__DIR__).'/tripay/Tripay.php';

function tripay_verify_callback($reference, array $expectedStatus = [])
{
    global $gatewayParams;

    if ($gatewayParams['verify_callback'] != '1') {
        return true;
    }

    $baseurl = (isset($gatewayParams['mode']) && $gatewayParams['mode'] == 'production')
        ? \Tripay\Core::$production_baseurl
        : \Tripay\Core::$sandbox_baseurl;

    $url = rtrim($baseurl, '/').'/transaction/detail?reference='.$reference;
    
    try {
        $result = \Tripay\ApiRequestor::remoteCall($url, [], false, $gatewayParams['apiKey']);
    } catch (\Exception $e) {
        die($e->getMessage());
    }

    if (empty($expectedStatus) || in_array(strtoupper($result->data->status), $expectedStatus)) {
        return true;
    }

    return false;
}

$json = file_get_contents("php://input");

if (empty($json)) {
    exit("Invalid JSON");
}

$data = json_decode($json);

$paymentCode = isset($data->payment_method_code) ? strtoupper($data->payment_method_code) : '';

$module = \Tripay\Core::modules($paymentCode);

if (is_null($module)) {
    logTransaction($paymentCode, "Unknown payment method", "TriPay Callback Invalid");
    exit("Unknown payment method");
}

$path = dirname(__DIR__).'/'.$module['gateway'].'.php';

if (!file_exists($path)) {
    logTransaction($module['gateway'], "Module not available", "TriPay Callback Invalid");
    exit("Module not available");
}

$gatewayParams = getGatewayVariables($module['gateway']);

if (!$gatewayParams['type'] || !isset($gatewayParams['type'])) {
    logTransaction($module['gateway'], "Module not enabled", "TriPay Callback Invalid");
    exit("Module not enabled");
}

$callbackSignature = isset($_SERVER['HTTP_X_CALLBACK_SIGNATURE']) ? $_SERVER['HTTP_X_CALLBACK_SIGNATURE'] : '';
$signature = hash_hmac("sha256", $json, $gatewayParams["privateKey"]);

if (!hash_equals($signature, $callbackSignature)) {
    logTransaction($module['gateway'], "Invalid Signature: incoming(".$callbackSignature.") vs local(".$signature.")", "TriPay Callback Invalid");
    exit("Invalid Signature");
}

$event = isset($_SERVER["HTTP_X_CALLBACK_EVENT"]) ? $_SERVER["HTTP_X_CALLBACK_EVENT"] : '';
if ($event == 'payment_status') {
    $reference = $data->reference;
    $order_id = $data->merchant_ref;
    $paymentAmount = $data->total_amount - $data->fee_customer;

    if ($data->status == 'PAID') {
        $order_id = checkCbInvoiceID($order_id, $gatewayParams['name']);

        checkCbTransID($reference);

        if (tripay_verify_callback($reference, ['PAID'])) {
            addInvoicePayment(
                $order_id,
                $reference,
                $paymentAmount,
                0,
                $module['gateway']
            );

            echo json_encode(["success" => true, "message" => "Order ".$order_id." has been marked as PAID"]);
            exit;
        } else {
            die("Callback verification failed");
        }
    } else {
        die("No action was taken. Current callback status is: ".$data->status);
    }
}
