<?php

namespace Tripay;

defined("WHMCS") or die("This file cannot be accessed directly");

use Exception;
use WHMCS\Utility\Environment\WebHelper;

class Core
{
    public static $production_baseurl = 'https://tripay.co.id/api';
    public static $sandbox_baseurl = 'https://tripay.co.id/api-sandbox';

    public static function system_path($path = '')
    {
        $sp = dirname(__DIR__, 4);
        if (!empty($path)) {
            $sp .= '/' . ltrim($path, '/');
        }
        return $sp;
    }

    public static function site_url($path = '')
    {
        $protocol = self::getThisUrlProtocol();
        $hostname = self::getThisUrlHost();
        $base = WebHelper::getBaseUrl();

        $url = rtrim($protocol . '://' . $hostname, '/');
        if (!empty($base) && $base != '/') {
            $url .= '/' . trim($base, '/');
        }
        if (!empty($path)) {
            $url .= '/' . ltrim($path, '/');
        }

        return $url;
    }

    public static function modules($code = null)
    {
        $modules = [
            'PERMATAVA' => [
                'gateway'   => 'tripay_permata_va',
                'icon'      => 'permata-va.png',
                'type'      => 'DIRECT'
            ],
            'BRIVA' => [
                'gateway'   => 'tripay_bri_va',
                'icon'      => 'bri-va.png',
                'type'      => 'DIRECT'
            ],
            'BNIVA' => [
                'gateway'   => 'tripay_bni_va',
                'icon'      => 'bni-va.png',
                'type'      => 'DIRECT'
            ],
            'BCAVA' => [
                'gateway'   => 'tripay_bca_va',
                'icon'      => 'bca-va.png',
                'type'      => 'DIRECT'
            ],
            'MANDIRIVA' => [
                'gateway'   => 'tripay_mandiri_va',
                'icon'      => 'mandiri-va.png',
                'type'      => 'DIRECT'
            ],
            'MYBVA' => [
                'gateway'   => 'tripay_maybank_va',
                'icon'      => 'maybank-va.png',
                'type'      => 'DIRECT'
            ],
            'SMSVA' => [
                'gateway'   => 'tripay_sms_va',
                'icon'      => 'sms-va.png',
                'type'      => 'DIRECT'
            ],
            'MUAMALATVA' => [
                'gateway'   => 'tripay_muamalat_va',
                'icon'      => 'muamalat-va.png',
                'type'      => 'DIRECT'
            ],
            'CIMBVA' => [
                'gateway'   => 'tripay_cimb_va',
                'icon'      => 'cimb-va.png',
                'type'      => 'DIRECT'
            ],
            'SAMPOERNAVA' => [
                'gateway'   => 'tripay_sampoerna_va',
                'icon'      => 'sampoerna-va.png',
                'type'      => 'DIRECT'
            ],
            'INDOMARET' => [
                'gateway'   => 'tripay_indomaret',
                'icon'      => 'indomaret.png',
                'type'      => 'DIRECT'
            ],
            'ALFAMART' => [
                'gateway'   => 'tripay_alfamart',
                'icon'      => 'alfamart.png',
                'type'      => 'DIRECT'
            ],
            'ALFAMIDI' => [
                'gateway'   => 'tripay_alfamidi',
                'icon'      => 'alfamidi.png',
                'type'      => 'DIRECT'
            ],
            'QRIS' => [
                'gateway'   => 'tripay_qris',
                'icon'      => 'qris.png',
                'type'      => 'DIRECT'
            ],
            'QRISC' => [
                'gateway'   => 'tripay_qrisc',
                'icon'      => 'qris.png',
                'type'      => 'DIRECT'
            ],
            'QRIS2' => [
                'gateway'   => 'tripay_qris2',
                'icon'      => 'qris.png',
                'type'      => 'DIRECT'
            ],
            'CC' => [
                'gateway'   => 'tripay_cc',
                'icon'      => 'cc.png',
                'type'      => 'REDIRECT'
            ],
            'OVO' => [
                'gateway'   => 'tripay_ovo',
                'icon'      => 'ovo.png',
                'type'      => 'REDIRECT'
            ],
            'SHOPEEPAY' => [
                'gateway'   => 'tripay_shopeepay',
                'icon'      => 'shopeepay.png',
                'type'      => 'REDIRECT'
            ],
            'BSIVA' => [
                'gateway'   => 'tripay_bsi_va',
                'icon'      => 'bsi-va.png',
                'type'      => 'DIRECT'
            ],
            'OCBCVA' => [
                'gateway'   => 'tripay_ocbc_va',
                'icon'      => 'ocbc-va.png',
                'type'      => 'DIRECT'
            ],
            'DANAMONVA' => [
                'gateway'   => 'tripay_danamon_va',
                'icon'      => 'danamon-va.png',
                'type'      => 'DIRECT'
            ],
        ];

        $code = strtoupper($code);

        return $code ? (isset($modules[$code]) ? $modules[$code] : null) : $modules;
    }

    public static function getRedirectionUrl($baseUrl, $params, $apiKey = "")
    {
        $result = ApiRequestor::post(rtrim($baseUrl, '/') . '/transaction/create', $params, $apiKey);
        return $result->data->checkout_url;
    }

    public static function requestTransaction($baseUrl, $params, $apiKey = "")
    {
        $result = ApiRequestor::post(rtrim($baseUrl, '/') . '/transaction/create', $params, $apiKey);
        return $result->data;
    }

    public static function config($title = "")
    {
        return array(
            'FriendlyName' => array(
                'Type' => 'System',
                'Value' => $title,
            ),
            'mode' => array(
                'FriendlyName' => 'Mode Integrasi',
                'Type' => 'dropdown',
                'Size' => '25',
                'Default' => 'sandbox',
                'Description' => '<br><small><b>Sandbox</b> digunakan untuk masa pengembangan<br/><b>Production</b> digunakan untuk transaksi riil</small>',
                'Options' => array(
                    'sandbox' => 'Sandbox',
                    'production' => 'Production',
                )
            ),
            'merchantCode' => array(
                'FriendlyName' => 'Merchant Code',
                'Type' => 'text',
                'Size' => '50',
                'Default' => '',
                'Description' => '<br><small>Untuk mode <b>Sandbox</b> lihat <a href="https://tripay.co.id/simulator/merchant" target="_blank" style="text-decoration:underline">di sini</a><br/>Untuk mode <b>Production</b> lihat <a href="https://tripay.co.id/member/merchant" target="_blank" style="text-decoration:underline">di sini</a></small>',
            ),
            'apiKey' => array(
                'FriendlyName' => 'API Key',
                'Type' => 'text',
                'Size' => '64',
                'Default' => '',
                'Description' => '<br><small>Untuk mode <b>Sandbox</b> lihat <a href="https://tripay.co.id/simulator/merchant" target="_blank" style="text-decoration:underline">di sini</a><br/>Untuk mode <b>Production</b> lihat <a href="https://tripay.co.id/member/merchant" target="_blank" style="text-decoration:underline">di sini</a> lalu klik tombol <b>Opsi > Edit</b></small>',
            ),
            'privateKey' => array(
                'FriendlyName' => 'Private Key',
                'Type' => 'text',
                'Size' => '64',
                'Default' => '',
                'Description' => '<br><small>Untuk mode <b>Sandbox</b> lihat <a href="https://payment.tripay.co.id/simulator/merchant" target="_blank" style="text-decoration:underline">di sini</a><br/>Untuk mode <b>Production</b> lihat <a href="https://tripay.co.id/member/merchant" target="_blank" style="text-decoration:underline">di sini</a> lalu klik tombol <b>Opsi > Edit</b></small>',
            ),
            'expired' => array(
                'FriendlyName' => 'Masa Aktif',
                'Type' => 'dropdown',
                'Size' => '25',
                'Default' => '1',
                'Description' => '<br><small>Masa Berlaku Kode Bayar/Nomor VA</small>',
                'Options' => array(
                    '1' => '1 Hari',
                    '2' => '2 Hari',
                    '3' => '3 Hari',
                    '4' => '4 Hari',
                    '5' => '5 Hari',
                    '6' => '6 Hari',
                    '7' => '7 Hari',
                    '8' => '8 Hari',
                    '9' => '9 Hari',
                    '10' => '10 Hari',
                    '11' => '11 Hari',
                    '12' => '12 Hari',
                    '13' => '13 Hari',
                    '14' => '14 Hari',
                )
            ),
            'checkout_type' => array(
                'FriendlyName' => 'Tipe Checkout',
                'Type' => 'dropdown',
                'Size' => '25',
                'Default' => 'REDIRECT',
                'Description' => '',
                'Options' => array(
                    'REDIRECT' => 'REDIRECT',
                    'DIRECT' => 'DIRECT'
                )
            ),
            'verify_callback' => array(
                'FriendlyName' => 'Verifikasi Data Callback',
                'Type' => 'dropdown',
                'Size' => '25',
                'Default' => '0',
                'Description' => '<br><small>Aktifkan untuk mode Production dan nonaktifkan untuk mode Sandbox</small>',
                'Options' => array(
                    '0' => 'Tidak Aktif',
                    '1' => 'Aktif'
                )
            ),
            'callback' => array(
                'FriendlyName' => 'Callback URL',
                'Description' => '<b>' . self::callback_url() . '</b><br><small>Masukan URL ini ke kolom Callback URL, di <a href="https://tripay.co.id/member/merchant" target="_blank" style="text-decoration:underline">https://tripay.co.id/member/merchant</a> lalu klik tombol <b>Opsi > Edit</b> sesuai merchant anda</small>'
            )
        );
    }

    public static function return_url()
    {
        $protocol = self::getThisUrlProtocol();
        $hostname = self::getThisUrlHost();

        return rtrim($protocol . '://' . $hostname, '/') . $_SERVER['REQUEST_URI'];
    }

    public static function callback_url()
    {
        return self::site_url('/modules/gateways/callback/tripay.php');
    }

    public static function link($params, $method_code)
    {
        if (!isset($_GET['id'])) {
            return false;
        }

        $module = self::modules($method_code);

        if (is_null($module)) {
            throw new Exception("Failed to generate payment link: Module not available");
        }

        $invoice = localAPI('GetInvoice', ['invoiceid' => $params['invoiceid']]);

        if (isset($invoice['status']) && strtolower($invoice['status']) != 'unpaid') {
            return false;
        }

        $merchant_code = $params['merchantCode'];
        $amount = (int) $params['amount'];
        $order_id = $params['invoiceid'];
        $apiKey = $params['apiKey'];
        $privateKey = $params['privateKey'];
        $baseurl = (isset($params['mode']) && $params['mode'] == 'production') ? self::$production_baseurl : self::$sandbox_baseurl;
        $expired = (int) $params['expired'];
        $firstname = $params['clientdetails']['firstname'];
        $lastname = $params['clientdetails']['lastname'];
        $email = $params['clientdetails']['email'];
        $phoneNumber = $params['clientdetails']['phonenumber'];
        $langPayNow = $params['langpaynow'];
        $checkout_type = isset($params['checkout_type']) ? $params['checkout_type'] : 'REDIRECT';

        $ProducItem = array(
            'name' => $params["description"],
            'price' => $amount,
            'quantity' => 1
        );

        $item_details = array($ProducItem);

        $signature = hash_hmac("sha256", $merchant_code . $order_id . $amount, $privateKey);

        if (!in_array(substr($phoneNumber, 0, 1), ['0', '+'])) {
            $phoneNumber = '0' . $phoneNumber;
        }

        $expired = $expired <= 0 ? 1 : $expired;

        $params = array(
            "amount" => $amount,
            "method" => $method_code,
            "merchant_ref" => $order_id,
            "customer_name" => $firstname . " " .  $lastname,
            "customer_email" => $email,
            "customer_phone" => $phoneNumber,
            "expired_time" => (time() + (24 * 60 * 60 * intval($expired))),
            "return_url" => self::return_url(),
            "order_items" => $item_details,
            "signature" => $signature,
        );

        try {
            $trx = self::requestTransaction($baseurl, $params, $apiKey);
            $redirUrl = !empty($trx->pay_url) ? $trx->pay_url : $trx->checkout_url;
            $payCode = $trx->pay_code;
            $qrUrl = isset($trx->qr_url) ? $trx->qr_url : null;
            $expiredTime = $trx->expired_time;

            switch (date_default_timezone_get()) {
                case 'Asia/Jakarta':
                    $tz = 'WIB';
                    break;

                case 'Asia/Makassar':
                case 'Asia/Pontianak':
                    $tz = 'WITA';
                    break;

                case 'Asia/Jayapura':
                    $tz = 'WIT';
                    break;

                default:
                    $tz = '';
                    break;
            }
        } catch (Exception $e) {
            error_log(__FILE__ . '(' . __LINE__ . '): ' . $e->getMessage() . PHP_EOL, 0);
            throw new Exception("Failed to generate payment link: " . $e->getMessage());
        }

        $iconurl = self::site_url('/modules/gateways/tripay/assets/' . $module['icon']);

        $htmlOutput = "";

        if ($iconurl) {
            $htmlOutput .= '<img src="' . $iconurl . '" style="height:100%;max-height:40px" /><br><br>';
        }

        if ($checkout_type != 'DIRECT' || $module['type'] == 'REDIRECT') {
            $htmlOutput .= '<button class="btn btn-success btn-sm" onclick="javascript:window.location.href=\'' . $redirUrl . '\'">' . $langPayNow . '</button>';
        } else {
            if (!empty($qrUrl)) {
                $htmlOutput .= '<a href="' . $qrUrl . '" target="_blank"><img src="' . $qrUrl . '" style="width:100%;max-width:150px" /></a><br/>';
            } else {
                $htmlOutput .= 'Kode Bayar/No. VA : <b>' . $payCode . '</b><br/>';
            }
            $htmlOutput .= 'Batas Pembayaran : <b>' . strftime('%d %B %Y %H:%M', $expiredTime) . ' ' . $tz . '</b>';
        }

        return $htmlOutput;
    }

    public static function getThisUrlProtocol()
    {
        if (isset($_SERVER['HTTP_X_FORWARDED_PROTO'])) {
            if ($_SERVER['HTTP_X_FORWARDED_PROTO'] == 'https') {
                $_SERVER['HTTPS']       = 'on';
                $_SERVER['SERVER_PORT'] = 443;
            }
        }
        $protocol = 'http';
        if (isset($_SERVER['HTTPS'])) {
            $protocol = (($_SERVER['HTTPS'] == 'on') ? 'https' : 'http');
        } else {
            $protocol = (isset($_SERVER["SERVER_PROTOCOL"]) ? $_SERVER["SERVER_PROTOCOL"] : 'http');
            $protocol = ((strtolower(substr($protocol, 0, 5)) == 'https') ? 'https' : 'http');
        }
        return $protocol;
    }

    public static function getThisUrlHost()
    {
        $currentPath = (isset($_SERVER['SCRIPT_NAME']) ? $_SERVER['SCRIPT_NAME'] : '');
        $pathInfo = pathinfo(dirname($currentPath));
        $hostName = (isset($_SERVER['HTTP_HOST']) ? $_SERVER['HTTP_HOST'] : '');
        return $hostName;
    }
}
